package eu.asvachin.audiorouter;

import java.util.Arrays;

/**
 * Just a wrapper around a byte array.
 *
 * @author Miles Asvachin
 */
public class Buffer
{
	private final byte[] data;
	private boolean hasData;
	
	public Buffer(final int newSize)
	{
		data = new byte[newSize];
		hasData = false;
	}
	
	public int getSize()
	{
		return data.length;
	}
	
	public synchronized void reset()
	{
		if(hasData)
		{
			Arrays.fill(data, (byte)0);
			hasData = false;
			notifyAll();
		}
	}
	
	public synchronized void write(final byte[] newData)
	{
		try
		{
			while(hasData)
			{
				wait();
			}
			
			System.arraycopy(newData, 0, data, 0, newData.length);
			hasData = true;
			notifyAll();
		}
		catch(InterruptedException interruptedException)
		{
		}
	}
	
	public synchronized void read(final byte[] outData)
	{
		try
		{
			while(!hasData)
			{
				wait();
			}
			
			System.arraycopy(data, 0, outData, 0, data.length);
			hasData = false;
			notifyAll();
		}
		catch(InterruptedException interruptedException)
		{
		}
	}
}
