package eu.asvachin.audiorouter;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.TargetDataLine;

/**
 * Reads from a TargetDataLine.
 *
 * @author Miles Asvachin
 */
public final class InputReader extends EndPoint<TargetDataLine>
{
	private final TargetDataLine input;
	
	/**
	 * Creates a thread running an InputReader.
	 * 
	 * @param newMixerInfo the mixer to get a TargetDataLine from.
	 * @param newFormat the format to open the line with.
	 * @param newBuffer the buffer to write data to.
	 * @return the new InputReader.
	 */
	public static InputReader createReader(final Mixer.Info newMixerInfo, final AudioFormat newFormat, final Buffer newBuffer) throws LineUnavailableException
	{
		final InputReader reader = new InputReader(TargetDataLine.class, newMixerInfo, newFormat, newBuffer);
		final Thread thread = new Thread(reader, "Input");
		thread.setPriority(java.lang.Thread.MAX_PRIORITY);
		thread.setDaemon(true);
		thread.start();
		
		return reader;
	}

	private InputReader(final Class<TargetDataLine> lineClass, final Mixer.Info newMixerInfo, final AudioFormat newFormat, final Buffer newBuffer) throws LineUnavailableException
    {
		super(lineClass, newMixerInfo, newFormat, newBuffer);
		
		input = line;
		input.open(newFormat, data.length);
		input.start();
    }

	@Override
	protected void transfer()
	{
		input.read(data, 0, data.length);
		buffer.write(data);
	}
}
