package eu.asvachin.audiorouter;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.SourceDataLine;

/**
 * Writes to a SourceDataLine.
 * 
 * @author Miles Asvachin
 */
public final class OutputWriter extends EndPoint<SourceDataLine>
{
	private final SourceDataLine output;

	/**
	 * Creates a thread running an OutputWriter.
	 * 
	 * @param newMixerInfo the mixer to get a SourceDataLine from.
	 * @param newFormat the format to open the line with.
	 * @param newBuffer the buffer to read data from.
	 * @return the new OutputWriter.
	 */
	public static OutputWriter createWriter(final Mixer.Info newMixerInfo, final AudioFormat newFormat, final Buffer newBuffer) throws LineUnavailableException
	{
		final OutputWriter writer = new OutputWriter(SourceDataLine.class, newMixerInfo, newFormat, newBuffer);
		final Thread thread = new Thread(writer, "Output");
		thread.setPriority(java.lang.Thread.MAX_PRIORITY);
		thread.setDaemon(true);
		thread.start();
		
		return writer;
	}
	
	private OutputWriter(final Class<SourceDataLine> lineClass, final Mixer.Info newMixerInfo, final AudioFormat newFormat, final Buffer newBuffer) throws LineUnavailableException
	{
		super(lineClass, newMixerInfo, newFormat, newBuffer);
		
		output = line;
		output.open(newFormat, data.length);
		output.start();
	}

	@Override
	protected void transfer()
	{
		buffer.read(data);
		output.write(data, 0, data.length);
	}
}
